function Threshold = AdaptiveOtsu(OrigImage)

GlobalThreshold = Otsu(OrigImage);

[m,n] = size(OrigImage);

BlockSize = max(50,min(round(m/20),round(n/20)));

%%% Calculates a range of acceptable block sizes as plus-minus 10%
%%% of the suggested block size.
BlockSizeRange = floor(1.1*BlockSize):-1:ceil(0.9*BlockSize);
[ignore,index] = min(ceil(m./BlockSizeRange).*BlockSizeRange-m + ceil(n./BlockSizeRange).*BlockSizeRange-n); %#ok Ignore MLint
BestBlockSize = BlockSizeRange(index);
%%% Pads the image so that the blocks fit properly.
RowsToAdd = BestBlockSize*ceil(m/BestBlockSize) - m;
ColumnsToAdd = BestBlockSize*ceil(n/BestBlockSize) - n;
RowsToAddPre = round(RowsToAdd/2);
RowsToAddPost = RowsToAdd - RowsToAddPre;
ColumnsToAddPre = round(ColumnsToAdd/2);
ColumnsToAddPost = ColumnsToAdd - ColumnsToAddPre;
PaddedImage = padarray(OrigImage,[RowsToAddPre ColumnsToAddPre],'replicate','pre');
PaddedImage = padarray(PaddedImage,[RowsToAddPost ColumnsToAddPost],'replicate','post');
PaddedImageandCropMask = PaddedImage;               

Block = [BestBlockSize BestBlockSize];
Threshold = blkproc(PaddedImageandCropMask,Block,@Otsu);
warning off MATLAB:divideByZero
Threshold = imresize(Threshold, size(PaddedImage), 'bilinear');
Threshold = Threshold(RowsToAddPre+1:end-RowsToAddPost,ColumnsToAddPre+1:end-ColumnsToAddPost);
warning on MATLAB:divideByZero

% Threshold(Threshold <= 0.7*GlobalThreshold) = 0.7*GlobalThreshold;
% Threshold(Threshold >= 1.5*GlobalThreshold) = 1.5*GlobalThreshold;
