function [MaximaImage,MinDiameter,MaxDiameter] = CenterOfCells(OrigImage,MinDiameter,MaxDiameter,coefficient)
% Finds the local maximus on the image
% The input variables are
% OrigGrai is grayscale image which should be analysed
% MinDiameter,MaxDiameter are minimum and maximus diameters of nuclei



Threshold = AdaptiveOtsu(OrigImage);

[len_x len_y len_c] = size(OrigImage);
BW = ones(len_x,len_y);
for i=1:len_x
    for j=1:len_y
        if OrigImage(i,j)<Threshold(i,j)
            BW(i,j) = 0;
        end
    end
end
[B L] = bwboundaries(BW,'noholes');
stats = regionprops(L,'Area','PixelIdxList','EquivDiameter');
MinArea = fix(0.25*pi*MinDiameter*MinDiameter);
BW = bwareaopen(BW,MinArea);

SizeOfSmoothingFilter = 1*MinDiameter;

BlurredImage = CPsmooth(OrigImage,'Gaussian Filter',SizeOfSmoothingFilter,0);
BlurredImage = im2double(BlurredImage);

ImageResizeFactor = 10/MinDiameter;
MaximaSuppresionSize = 10;
MaximaMask = getnhood(strel('disk',MaximaSuppresionSize));

ResizedBlurredImage = imresize(BlurredImage,ImageResizeFactor,'bilinear');
MaximaImage = ResizedBlurredImage;
MaximaImage(ResizedBlurredImage < ...
                        ordfilt2(ResizedBlurredImage,sum(MaximaMask(:)),MaximaMask)) = 0;
                    MaximaImage = imresize(MaximaImage,size(BlurredImage),'bilinear');
MaximaImage = MaximaImage > Threshold;
if all(MaximaImage(:))
    MaximaImage = zeros(size(MaximaImage));
else
    MaximaImage = bwmorph(MaximaImage,'shrink',inf);
end
