function level = Otsu(im)
% find global threshold of input image
im = im2double(im);
if ndims(im) == 3
    Image = im(:,:,1);
    CropMask = im(:,:,2);
    clear im
    im = Image(CropMask==1);
else im = im(:);
end

if max(im) == min(im)
    level = im(1);
elseif isempty(im)
    %%% im will be empty if the entire image is cropped away by the
    %%% CropMask. I am not sure whether it is better to then set the level
    %%% to 0 or 1. Setting the level to empty causes problems downstream.
    %%% Presumably setting the level to 1 will not cause major problems
    %%% because the other blocks will average it out as we get closer to
    %%% real objects?
    level = 1;
else
    %%% We want to limit the dynamic range of the image to 256. Otherwise,
    %%% an image with almost all values near zero can give a bad result.
    minval = max(im)/256;
    im(im < minval) = minval;
%     im = double(im);
    im = log(im);
    minval = min (im);
    maxval = max (im);
    im = (im - minval) / (maxval - minval);
%     level = graythresh(im);
    level = exp(minval + (maxval - minval) * graythresh(im));
end 