function [NUCLEI_SINGLED,vx,vy] = nuclei_segmentation(I,TUMOR,MinDiameter,MaxDiameter,BW_3)

MaxArea = pi*(0.5*MaxDiameter)^2; 
% addaptive thresholding of the image
    D = bwdist(BW_3);
    DL = watershed(D);
    bgm = DL == 0;
    bgm = ~bgm;
    bgm = and(TUMOR,bgm);
   
    [B_REGION,L_REGION] = bwboundaries(bgm,'noholes');
    stats = regionprops(L_REGION,'PixelIdxList');
    BLUE = I(:,:,3);
    [len_x, len_y, len_z] = size(I);
    NUCLEI = logical(zeros(len_x, len_y));
    for i=1:length(B_REGION)
        pixels = [];
        pixels = stats(i).PixelIdxList;
        array = [];
        array = BLUE(pixels(:));
        threshold = graythresh(array);
  
        indexes = [];
        indexes = find(array>threshold);
        NUCLEI(pixels(indexes(:))) = 1;
    end
    
    se = strel('disk', 2);
    NUCLEI = imdilate(NUCLEI,se);
    NUCLEI = imerode(NUCLEI,se);
    NUCLEI = imfill(NUCLEI,'holes');
    [B,L] = bwboundaries(NUCLEI,'noholes');
    imwrite(NUCLEI,['raw_nuclei.tif'])

% makes two masks of the nuclei. The first mask contains the normal objects. The second mask is used for the clumped objects    
    stats = regionprops(L,'Area','PixelIdxList','EquivDiameter');
    NUCLEI_CLUMPED = NUCLEI;
    NUCLEI_CLUMPED(:,:) = 0;
    NUCLEI_SINGLED= NUCLEI;
    
    for i=1:length(B)
        if stats(i).Area>MaxArea
            NUCLEI_CLUMPED(stats(i).PixelIdxList) = 1;
            NUCLEI_SINGLED(stats(i).PixelIdxList) = 0;
        end
    end
    NUCLEI_SINGLED = bwareaopen(NUCLEI_SINGLED,fix(0.25*pi*MinDiameter^2));

    [B_CLUMPED,L_CLUMPED] = bwboundaries(NUCLEI_CLUMPED,'noholes');
    stats_clumped = regionprops(L_CLUMPED,'Area','PixelIdxList');
   
    NUCLEI3 = NUCLEI_CLUMPED;
    se = strel('disk', 2);
    NUCLEI3 = imerode(NUCLEI3,se);
 
%     makes devision of clumped cells only for the clumped objects
    k = 0;
    MAX = sum(NUCLEI3(:));
    while MAX>MaxArea
        k = k+1
        WatershedBoundaries = []; BLUE_2 = []; Overlaid = [];
        BLUE_2 = bwdist(~NUCLEI_CLUMPED);
        Overlaid = imimposemin(1-BLUE_2,NUCLEI3);
        WatershedBoundaries = watershed(Overlaid) > 0;
        NUCLEI_CLUMPED = NUCLEI_CLUMPED.*WatershedBoundaries;
        B_CLUMPED = []; L_CLUMPED = []; stats_clumped = [];
        [B_CLUMPED,L_CLUMPED] = bwboundaries(NUCLEI_CLUMPED,'noholes');
        stats_clumped = regionprops(L_CLUMPED,'Area','PixelIdxList');
    
        for j=1:length(B_CLUMPED)
            if stats_clumped(j).Area<MaxArea
                NUCLEI_SINGLED(stats_clumped(j).PixelIdxList) = 1;
                NUCLEI_CLUMPED(stats_clumped(j).PixelIdxList) = 0;
            end
        end
        NUCLEI3 = imerode(NUCLEI3,se);    
        [B_NUCLEI3 L_NUCLEI3] = bwboundaries(NUCLEI3,'noholes');
        stats_NUCLEI3 = regionprops(L_NUCLEI3,'Area');
        AREA_L_NUCLEI3 = [];
        for f=1:length(B_NUCLEI3)
            AREA_L_NUCLEI3(f) = stats_NUCLEI3(f).Area;
        end
        MAX = max(AREA_L_NUCLEI3)
        sum(NUCLEI3(:))
    end

    NUCLEI_SINGLED2 = NUCLEI_SINGLED;
 
    %     NUCLEI_SINGLED = NUCLEI_SINGLED2;

    B_CLUMPED = []; L_CLUMPED = []; stats_clumped = [];
    [B_CLUMPED,L_CLUMPED] = bwboundaries(NUCLEI_SINGLED,'noholes');
    stats_clumped = regionprops(L_CLUMPED,'Area','PixelIdxList','Perimeter');
    AREA = [];
    for j=1:length(B_CLUMPED)
        AREA(j) = (stats_clumped(j).Perimeter^2)/stats_clumped(j).Area;
    end
    threshold = max(AREA)*graythresh(AREA/max(AREA));

    indexes = [];
    BLUE_2  = bwdist(~NUCLEI_CLUMPED);

     
    [CENTERS] = CenterOfCells(BLUE_2,MinDiameter,MaxDiameter,1);
    [s] = size(CENTERS);
    indexes = find(CENTERS>0)
    [I,J] = ind2sub(s,indexes)
     [vx, vy] = voronoi(J,I);
     
     s = size(NUCLEI_SINGLED);
    EDGE = zeros(s);
    se = strel('disk',1);
 
    for j=1:length(vx)
        
        N = fix(abs(vx(2,j)-vx(1,j)))+fix(abs(vy(2,j)-vy(1,j)));
        N=2*N;
        x = (linspace(vx(1,j),vx(2,j),N));
        
        a = (vy(2,j)-vy(1,j))/(vx(2,j)-vx(1,j));
        b = vy(2,j) - a*vx(2,j);
        y = fix(a*x+b);
        x = fix(x);
        indexes = [];
        indexes = find(x<1);
        x(indexes) = []; y(indexes) = [];
        indexes = [];
        indexes = find(y<1);
        x(indexes) = []; y(indexes) = [];
        indexes = [];
        indexes = find(y>s(2));
        x(indexes) = []; y(indexes) = [];
        indexes = [];
        indexes = find(x>s(1));
        x(indexes) = []; y(indexes) = [];        
        
        for k=1:length(x)
            NUCLEI_CLUMPED(y(k),x(k)) = 0;
            EDGE(y(k),x(k)) = 1;
%             NUCLEI_CLUMPED(y(k),x(k)-1) = 0;
%             NUCLEI_CLUMPED(y(k),x(k)+1) = 0;
        end
    end
        EDGE = imdilate(EDGE,se);

        NUCLEI_CLUMPED(EDGE>0) = 0;
    figure(3),imshow(NUCLEI_CLUMPED);
    figure(4),imshow(EDGE);

  
    NUCLEI_SINGLED  = or(NUCLEI_CLUMPED,NUCLEI_SINGLED);
    NUCLEI_SINGLED = bwareaopen(NUCLEI_SINGLED,fix(0.25*pi*MinDiameter^2));