function [NUCLEI_SINGLED,AREA] = nuclei_segmentation3(IMAGE,TUMOR,MinDiameter,MaxDiameter,BW_3)

MaxArea = pi*(0.5*MaxDiameter)^2; 
% addaptive thresholding of the image
    D = bwdist(BW_3);
    DL = watershed(D);
    bgm = DL == 0;
    bgm = ~bgm;
    bgm = and(TUMOR,bgm);
   
    [B_REGION,L_REGION] = bwboundaries(bgm,'noholes');
    stats = regionprops(L_REGION,'PixelIdxList');
    h = fspecial('gaussian',fix(MinDiameter),fix(MinDiameter/6));
    BLUE = imfilter(IMAGE(:,:,3),h);
%     BLUE = im2double(IMAGE(:,:,3));
    [len_x, len_y, len_z] = size(IMAGE);
    NUCLEI = logical(zeros(len_x, len_y));
    for i=1:length(B_REGION)
        pixels = [];
        pixels = stats(i).PixelIdxList;
        array = [];
        array = BLUE(pixels(:));
        threshold = graythresh(array);
  
        indexes = [];
        indexes = find(array>threshold);
        NUCLEI(pixels(indexes(:))) = 1;
    end
    
    se = strel('disk', 2);
    NUCLEI = imdilate(NUCLEI,se);
    NUCLEI = imerode(NUCLEI,se);
    NUCLEI = imfill(NUCLEI,'holes');
    [B,L] = bwboundaries(NUCLEI,'noholes');
    imwrite(NUCLEI,['raw_nuclei.tif'])

% makes two masks of the nuclei. The first mask contains the normal objects. The second mask is used for the clumped objects    
    stats = regionprops(L,'Area','PixelIdxList','EquivDiameter');
    NUCLEI_CLUMPED = NUCLEI;
    NUCLEI_CLUMPED(:,:) = 0;
    NUCLEI_SINGLED= NUCLEI;
    
    for i=1:length(B)
        if stats(i).Area>MaxArea
            NUCLEI_CLUMPED(stats(i).PixelIdxList) = 1;
            NUCLEI_SINGLED(stats(i).PixelIdxList) = 0;
        end
    end
    NUCLEI_SINGLED = bwareaopen(NUCLEI_SINGLED,fix(0.25*pi*MinDiameter^2));

    [B_CLUMPED,L_CLUMPED] = bwboundaries(NUCLEI_CLUMPED,'noholes');
    stats_clumped = regionprops(L_CLUMPED,'Area','PixelIdxList');
   
    NUCLEI3 = NUCLEI_CLUMPED;
    se = strel('disk', 2);
    NUCLEI3 = imerode(NUCLEI3,se);
 
%     makes devision of clumped cells only for the clumped objects
    k = 0;
    MAX = sum(NUCLEI3(:));
    while MAX>MaxArea
        k = k+1;
        WatershedBoundaries = []; BLUE_2 = []; Overlaid = [];
%         BLUE_2 = bwdist(~NUCLEI_CLUMPED);
        BLUE_2 = IMAGE(:,:,3);
        Overlaid = imimposemin(1-BLUE_2,NUCLEI3);
        WatershedBoundaries = watershed(Overlaid) > 0;
        NUCLEI_CLUMPED = NUCLEI_CLUMPED.*WatershedBoundaries;
        B_CLUMPED = []; L_CLUMPED = []; stats_clumped = [];
        [B_CLUMPED,L_CLUMPED] = bwboundaries(NUCLEI_CLUMPED,'noholes');
        stats_clumped = regionprops(L_CLUMPED,'Area','PixelIdxList');
    
        for j=1:length(B_CLUMPED)
            if stats_clumped(j).Area<MaxArea
                NUCLEI_SINGLED(stats_clumped(j).PixelIdxList) = 1;
                NUCLEI_CLUMPED(stats_clumped(j).PixelIdxList) = 0;
            end
        end
        NUCLEI3 = imerode(NUCLEI3,se);    
        [B_NUCLEI3 L_NUCLEI3] = bwboundaries(NUCLEI3,'noholes');
        stats_NUCLEI3 = regionprops(L_NUCLEI3,'Area');
        AREA_L_NUCLEI3 = [];
        for f=1:length(B_NUCLEI3)
            AREA_L_NUCLEI3(f) = stats_NUCLEI3(f).Area;
        end
        MAX = max(AREA_L_NUCLEI3);        
    end

    NUCLEI_SINGLED2 = NUCLEI_SINGLED;

    B_CLUMPED = []; L_CLUMPED = []; stats_clumped = [];
    [B_CLUMPED,L_CLUMPED] = bwboundaries(NUCLEI_SINGLED,'noholes');
    stats_clumped = regionprops(L_CLUMPED,'Area','PixelIdxList','Perimeter');
    AREA = [];
    for j=1:length(B_CLUMPED)
        AREA(j) = 4*pi*stats_clumped(j).Area/(stats_clumped(j).Perimeter^2);
    end
    threshold = mean(AREA);
    for i=1:length(B_CLUMPED)
        if (AREA(i)<threshold)&(stats_clumped(j).Area>MaxArea)
            NUCLEI_CLUMPED(stats_clumped(i).PixelIdxList) = 1;
            NUCLEI_SINGLED(stats_clumped(i).PixelIdxList) = 0;
        end
    end
        
    BLUE_2  = bwdist(~NUCLEI_CLUMPED);
    
%     BLUE_2 = 1-im2double(IMAGE(:,:,3));
%     BLUE_2(NUCLEI_CLUMPED<1)=0;
%       h = fspecial('gaussian',fix(MaxDiameter),fix(MaxDiameter/6));
%     BLUE_2 = imfilter(BLUE_2,h);
    
    NUCLEI_CLUMPED2 = NUCLEI_CLUMPED;
    NUCLEI_CLUMPED = voronoi_segmentation(BLUE_2,NUCLEI_CLUMPED,MinDiameter,MaxDiameter);     
    NUCLEI_SINGLED  = or(NUCLEI_CLUMPED,NUCLEI_SINGLED);
    
%      figure(2),imshow(NUCLEI_CLUMPED),title('NUCLEI_CLUMPED');
%      figure(3),imshow(NUCLEI_CLUMPED2),title('NUCLEI_CLUMPED_2');
     
     B_CLUMPED = []; L_CLUMPED = []; stats_clumped = [];
    [B_CLUMPED,L_CLUMPED] = bwboundaries(NUCLEI_CLUMPED,'noholes');
    stats_clumped = regionprops(L_CLUMPED,'Area','PixelIdxList','Perimeter');
    AREA = [];
    NUCLEI_CLUMPED(:,:) = 0;
    for j=1:length(B_CLUMPED)
        AREA(j) = stats_clumped(j).Area;
%         AREA(j) = 4*pi*stats_clumped(j).Area/(stats_clumped(j).Perimeter^2);
    end
    threshold = max(AREA)*graythresh((AREA)/max(AREA));
%     threshold = MaxArea;
    for i=1:length(B_CLUMPED)
        if (AREA(i)<threshold)
            NUCLEI_CLUMPED(stats_clumped(i).PixelIdxList) = 1;
            NUCLEI_SINGLED(stats_clumped(i).PixelIdxList) = 0;
        end
    end

    BLUE_2 = 1-im2double(IMAGE(:,:,3));
    BLUE_2(NUCLEI_CLUMPED<1)=0;
      h = fspecial('gaussian',fix(MaxDiameter),fix(MaxDiameter/6));
    BLUE_2 = imfilter(BLUE_2,h);
    figure(4),imshow(NUCLEI_CLUMPED),title('NUCLEI_CLUMPED');
   [NUCLEI_CLUMPED vx vy I J] = voronoi_segmentation(BLUE_2,NUCLEI_CLUMPED,MinDiameter,MaxDiameter);  
    
   threshold = fix(0.25*pi*MinDiameter*MinDiameter);
    NUCLEI_CLUMPED = bwareaopen(NUCLEI_CLUMPED,threshold);
    
     figure(5),imshow(BLUE_2,[]),title('NUCLEI_CLUMPED_2');
     hold on
     plot(vx,vy)
     plot(J,I,'p')
     hold off
     NUCLEI_CLUMPED3 = xor(NUCLEI_CLUMPED2,NUCLEI_CLUMPED); 
     figure(6),imshow(NUCLEI_CLUMPED),title('xor_NUCLEI_CLUMPED');
     hold on
     plot(vx,vy)
     plot(J,I,'p')
     hold off
    NUCLEI_SINGLED  = or(NUCLEI_CLUMPED,NUCLEI_SINGLED);
    
     B_CLUMPED = []; L_CLUMPED = []; stats_clumped = [];
    [B_CLUMPED,L_CLUMPED] = bwboundaries(NUCLEI_SINGLED,'noholes');
    stats_clumped = regionprops(L_CLUMPED,'Area','PixelIdxList','Perimeter');
    AREA = [];
    for i=1:length(B_CLUMPED)
        AREA(i) = stats_clumped(i).Area;
    end
    
%     threshold = fix(max(AREA)*graythresh(AREA/max(AREA)));
    threshold = fix(0.25*pi*MinDiameter*MinDiameter);
    NUCLEI_SINGLED = bwareaopen(NUCLEI_SINGLED,threshold);